import enum
from enum import Enum


class Ethnicity(str, enum.Enum):
    AFRICAN = "african"
    SOUTH_ASIAN = "south_asian"
    SOUTHEAST_ASIAN = "southeast_asian"
    HISPANIC = "hispanic"
    MEDITERRANEAN = "mediterranean"
    NORTH_EUROPEAN = "north_european"


class Gender(str, enum.Enum):
    FEMALE = "female"
    MALE = "male"


class Age(str, enum.Enum):
    ADULT = "adult"
    OLD = "old"
    YOUNG = "young"


class EyesColor(str, enum.Enum):
    BROWN = "brown"
    GREEN = "green"
    BLUE = "blue"
    HAZEL = "hazel"
    GRAY = "gray"
    AMBER = "amber"


class AccessoryPosition(str, enum.Enum):
    ON_CHIN = "chin"
    ON_MOUTH = "mouth"
    ON_NOSE = "nose"


class GlassesModel(str, enum.Enum):
    GENERAL = "general"


class GlassesBrand(str, enum.Enum):
    GENERAL = "general"


class GlassesStyle(str, enum.Enum):
    AVIATOR = "aviator"
    BROWLINE = "browline"
    CAT_EYE = "cat_eye"
    GEOMETRIC = "geometric"
    OVAL = "oval"
    OVERSIZED = "oversized"
    READING_FULL_FRAME = "reading_full_frame"
    READING_RIMLESS = "reading_rimless"
    ROUND = "round"


class MaskStyle(str, enum.Enum):
    CLOTH = "cloth"


class HairLength(str, enum.Enum):
    BUZZ_CUT = "buzz_cut"
    UNDEFINED = "undefined"
    SHOULDER = "shoulder"
    CHIN = "chin"
    ARMPIT = "armpit"
    EAR = "ear"
    TAILBONE = "tailbone"
    MID_BACK = "mid_back"


class HairStyle(str, enum.Enum):
    LAYERED = "layered"
    UNDEFINED = "undefined"
    HAIR_DOWN = "hair_down"
    BUN = "bun"
    CURTAIN = "curtain"
    BALDING = "balding"
    BANGS = "bangs"
    HIGH_TOP_CUT = "high_top_cut"
    PONYTAIL = "ponytail"
    PULLED_BACK = "pulled_back"
    AFRO = "afro"
    CREW_CUT = "crew_cut"
    BOB = "bob"


class Environment(str, enum.Enum):
    INDOOR = "indoor"
    OUTDOOR = "outdoor"
    CROSS_POLARIZED = "cross_polarized"


class TimeOfDay(str, enum.Enum):
    MORNING = "morning"
    EVENING = "evening"
    NIGHT = "night"
    NA = "N/A"


class Generator(str, enum.Enum):
    IDENTITIES = "identities"
    HIC = "hic"


class FacialHairStyle(str, enum.Enum):
    FULL_BEARD = "full_beard"
    STUBBLE = "stubble"
    MUSTACHE = "mustache"
    BEARD = "beard"
    PARTIAL_BEARD = "partial_beard"


class HICDomain(str, enum.Enum):
    IN_CABIN = "in_cabin"
    SMART_OFFICE = "smart_office"
    HOME_SECURITY = "home_security"
    SMART_FITNESS = "smart_fitness"


class HICSubDomain(str, enum.Enum):
    HOME_OFFICE = "home_office"
    HOME_FITNESS = "home_fitness"
    FRONT_DOOR_CAMERA = "front_door_camera"
    IN_CABIN = "in_cabin"


class HumanBehaviour(str, enum.Enum):
    BICEPS_CURLS_DUMBBELLS = "biceps_curls_dumbbells"
    BRINGING_MULTI_PACKAGES_TO_THE_FRONT_DOOR = "bringing_multi_packages_to_the_front_door"
    DRIVER_ADJUSTING_MIRROR = "driver_adjusting_mirror"
    DRIVER_DRESSING = "driver_dressing"
    DRIVER_DRINKING = "driver_drinking"
    DRIVER_EATING = "driver_eating"
    DRIVER_FACE_TOUCHING = "driver_face_touching"
    DRIVER_FALLING_ASLEEP = "driver_falling_asleep"
    DRIVER_LOOKING_AROUND = "driver_looking_around"
    DRIVER_PLAYING_WITH_MOBILE_PHONE = "driver_playing_with_mobile_phone"
    DRIVER_PUT_MAKEUP = "driver_put_makeup"
    DRIVER_SHAVING = "driver_shaving"
    DRIVER_SINGING = "driver_singing"
    DRIVER_SMOKING = "driver_smoking"
    DRIVER_STANDARD_DRIVING_ONE_HAND = "driver_standard_driving_one_hand"
    DRIVER_STANDARD_DRIVING_TWO_HANDS = "driver_standard_driving_two_hands"
    DRIVER_TURNING_AROUND = "driver_turning_around"
    FALLING_FRONT_STEPS = "falling_front_steps"
    FALLING_MEDICAL_CONDITION = "falling_medical_condition"
    FALLING_UNEVEN_ROAD = "falling_uneven_road"
    FORWARD_CRUNCHES = "forward_crunches"
    GLUTE_BRIDGE = "glute_bridge"
    LOOKING_AWAY_FROM_THE_LAPTOP = "looking_away_from_the_laptop"
    MIX_WITH_PACKAGES_AND_NON_DELIVERY_PACKAGES = "mix_with_packages_and_non_delivery_packages"
    OVERHEAD_SQUATS = "overhead_squats"
    PICKING_A_PACKAGE_FROM_THE_FRONT_DOOR = "picking_a_package_from_the_front_door"
    PLANK_FACE_DOWN = "plank_face_down"
    PUSH_UPS = "pushups"
    ROAD_RAGE = "road_rage"
    SLEEPY_CLOSING_EYES = "sleepy_closing_eyes"
    STANDING_UP_FROM_CHAIR_WALKING_TO_THE_SIDE = "standing_up_from_chair_walking_to_the_side"
    STATIC_LUNGE = "static_lunge"
    TALKING_FACE = "talking_face"
    TALKING_ON_MOBILE_PHONE = "talking_on_mobile_phone"
    TALKING_WHILE_TOUCHING_FACE = "talking_while_touching_face"
    TURNING_BACK_AND_LOOKING_AWAY_WITH_BODY = "turning_back_and_looking_away_with_body"
    WALKING_TO_THE_DOOR_WITH_A_DELIVERY_PACKAGE = "walking_to_the_door_with_a_delivery_package"
    WALKING_TO_THE_DOOR_WITH_PACKAGED_ITEMS = "walking_to_the_door_with_packaged_items"



from typing import List

from pydantic import validator

from datagen_protocol.schema.humans import Human, Accessories
from datagen_protocol.schema.environment import Camera, Light, Background, Wavelength
from datagen_protocol.schema import humans as humans_core_schema


class HumanDatapoint(humans_core_schema.HumanDatapoint):
    @validator("lights", always=True)
    def lights_and_not_nir_mutually_exclusive(cls, lights, values) -> List[Light]:
        has_lights = lights is not None and len(lights) > 0
        if has_lights and not cls._is_set_to_nir(values["camera"]):
            raise ValueError("Lights are only relevant if the camera is using 'nir' wavelength")
        return lights

    @validator("background", always=True)
    def background_and_nir_mutually_exclusive(cls, background, values) -> Background:
        if background is not None and cls._is_set_to_nir(values["camera"]):
            raise ValueError("Background is only relevant if the camera is not using 'nir' wavelength")
        return background

    @staticmethod
    def _is_set_to_nir(camera: Camera) -> bool:
        return camera.intrinsics.wavelength == Wavelength.NIR

    @validator("accessories", always=True)
    def mask_and_facial_hair_mutually_exclusive(cls, accessories, values: dict) -> Accessories:
        if accessories and accessories.mask and cls._has_facial_hair(values["human"]):
            raise ValueError("Facial hair and masks are mutually exclusive")
        return accessories

    @staticmethod
    def _has_facial_hair(human: Human) -> bool:
        return bool(human.head.facial_hair)



from enum import Enum
from typing import Optional

from datagen_protocol.config import conf
from datagen_protocol.schema import fields
from datagen_protocol.schema.d3 import Point, Rotation
from pydantic import BaseModel, Field


class Projection(str, Enum):
    PERSPECTIVE = "perspective"
    PANORAMIC = "panoramic"
    ORTHOGRAPHIC = "orthographic"


class Wavelength(str, Enum):
    VISIBLE = "visible"
    NIR = "nir"


class IntrinsicParams(BaseModel):
    projection: Projection = fields.enum(Projection, conf["camera"]["projection"])
    wavelength: Wavelength = fields.enum(Wavelength, conf["camera"]["wavelength"])
    resolution_height: int = fields.numeric(conf["camera"]["res"]["height"])
    resolution_width: int = fields.numeric(conf["camera"]["res"]["width"])
    fov_horizontal: int = fields.numeric(conf["camera"]["fov"]["horizontal"])
    fov_vertical: int = fields.numeric(conf["camera"]["fov"]["vertical"])
    sensor_width: float = fields.numeric(conf["camera"]["sensor"]["width"])
    fps: Optional[int] = None

    def dict(self, exclude_none=True, **kwargs):
        return super().dict(exclude_none=exclude_none, **kwargs)


class ExtrinsicParams(BaseModel):
    location: Point = fields.point(conf["camera"]["location"])
    rotation: Rotation = fields.rotation(conf["camera"]["rotation"])


class Camera(BaseModel):
    name: str = conf["camera"]["name"]
    extrinsic_params: ExtrinsicParams = Field(default_factory=ExtrinsicParams)
    intrinsic_params: IntrinsicParams = Field(default_factory=IntrinsicParams)

    @property
    def extrinsics(self) -> ExtrinsicParams:
        return self.extrinsic_params

    @extrinsics.setter
    def extrinsics(self, extrinsic_params: ExtrinsicParams) -> None:
        self.extrinsic_params = extrinsic_params

    @property
    def intrinsics(self) -> IntrinsicParams:
        return self.intrinsic_params

    @intrinsics.setter
    def intrinsics(self, intrinsic_params: IntrinsicParams) -> None:
        self.intrinsic_params = intrinsic_params



from typing import List, Union

from datagen_protocol.validation.humans.human import HumanDatapoint as ValidationHumanDatapoint
from datagen_protocol.schema import request as core_request_schema, DataSequence


class DataRequest(core_request_schema.DataRequest):
    datapoints: List[Union[DataSequence, ValidationHumanDatapoint]]


core_request_schema.DataRequest = DataRequest



from enum import Enum
from typing import Optional, List

from pydantic import Field, BaseModel

from datagen_protocol.schema.assets.attributes import (
    Age,
    Ethnicity,
    Gender,
    HairLength,
    FacialHairStyle,
    HairStyle,
    EyesColor,
)
from datagen_protocol.config import conf
from datagen_protocol.schema.assets.impl import Asset, AttributesList, AssetAttributes
from datagen_protocol.schema import fields
from datagen_protocol.schema.d3 import Point, Vector, Rotation
from datagen_protocol.schema.humans.accessories import Accessories
from datagen_protocol.schema.environment import Camera, Background, Light


class HairColor(BaseModel):
    melanin: float = fields.numeric(conf["human"]["hair"]["melanin"])
    redness: float = fields.numeric(conf["human"]["hair"]["redness"])
    whiteness: float = fields.numeric(conf["human"]["hair"]["whiteness"])
    roughness: float = fields.numeric(conf["human"]["hair"]["roughness"])
    index_of_refraction: float = fields.numeric(conf["human"]["hair"]["refraction"])


class HairModel(BaseModel):
    color_settings: HairColor = Field(default_factory=HairColor)


class HairAttributes(AssetAttributes):
    age_group_match: AttributesList[Age]
    ethnicity_match: AttributesList[Ethnicity]
    gender_match: AttributesList[Gender]
    length: HairLength
    style: AttributesList[HairStyle]


class FacialHairAttributes(AssetAttributes):
    style: FacialHairStyle


class EyebrowsAttributes(AssetAttributes):
    age_group_match: AttributesList[Age]
    ethnicity_match: AttributesList[Ethnicity]
    gender_match: AttributesList[Gender]


class Hair(HairModel, Asset[HairAttributes]):
    pass


class FacialHair(HairModel, Asset[FacialHairAttributes]):
    pass


class Eyebrows(HairModel, Asset[EyebrowsAttributes]):
    pass


class Gaze(BaseModel):
    distance: float = fields.numeric(conf["human"]["eyes"]["gaze"]["distance"])
    direction: Vector = fields.vector(conf["human"]["eyes"]["gaze"]["direction"])


class EyesAttributes(AssetAttributes):
    color: EyesColor


class Eyes(Asset[EyesAttributes]):
    target_of_gaze: Gaze = Field(default_factory=Gaze)
    eyelid_closure: float = fields.numeric(conf["human"]["eyes"]["eyelid_closure"])


class ExpressionName(str, Enum):
    NEUTRAL = "none"
    HAPPINESS = "happiness"
    SADNESS = "sadness"
    SURPRISE = "surprise"
    FEAR = "fear"
    ANGER = "anger"
    DISGUST = "disgust"
    CONTEMPT = "contempt"
    MOUTH_OPEN = "mouth_open"


class Expression(BaseModel):
    name: ExpressionName = fields.enum(ExpressionName, conf["human"]["expression"]["name"])
    intensity: float = fields.numeric(conf["human"]["expression"]["intensity"])


class Head(BaseModel):
    eyes: Eyes
    hair: Optional[Hair]
    eyebrows: Optional[Eyebrows]
    facial_hair: Optional[FacialHair]
    expression: Expression = Field(default_factory=Expression)
    location: Point = fields.point(conf["human"]["head"]["location"])
    rotation: Rotation = fields.rotation(conf["human"]["head"]["rotation"])


class HumanAttributes(AssetAttributes):
    ethnicity: Ethnicity
    age: Age
    gender: Gender


class Human(Asset[HumanAttributes]):
    head: Head


class HumanDatapoint(BaseModel):
    human: Human
    camera: Camera
    accessories: Optional[Accessories]
    background: Optional[Background]
    lights: Optional[List[Light]]



from enum import Enum
from typing import Union, List, TypeVar, Generic

from pydantic import BaseModel
from pydantic.generics import GenericModel
from pydantic.fields import ModelField, Field


A = TypeVar("A")


class AttributesList(list, Generic[A]):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v: Union[A, List[A]], field: ModelField) -> List[A]:
        if not isinstance(v, list):
            v = [v]
        return [field.type_(v_) for v_ in v]


AA = TypeVar("AA")

AP = TypeVar("AP")


class AssetAttributes(BaseModel):
    class Config:
        frozen = True


class Asset(GenericModel, Generic[AA]):
    id: str
    attributes: AA = Field(default=None)

    def dict(self, *args, **kwargs):
        kwargs["exclude"] = {"attributes"}
        return super().dict(*args, **kwargs)


class PresetAsset(GenericModel, Generic[AA, AP]):
    attributes: AA = Field(default=None)
    presets: AP = Field(default=None)

    def dict(self, *args, **kwargs):
        kwargs["exclude"] = {"attributes", "presets"}
        return super().dict(*args, **kwargs)



from datagen_protocol.schema.humans.accessories import (
    Accessories,
    FrameColor,
    Glasses,
    GlassesPosition,
    LensColor,
    Mask,
    MaskColor,
    MaskPosition,
    MaskTexture,
)
from datagen_protocol.schema.humans.human import (
    Expression,
    ExpressionName,
    Eyes,
    Eyebrows,
    FacialHair,
    Gaze,
    Hair,
    HairColor,
    Head,
    Human,
    HumanDatapoint,
)



import abc
from typing import TypeVar, Type

from pydantic import BaseModel, Field

from datagen_protocol.config import conf


class Coords3D(BaseModel):
    x: float = Field(ge=conf["d3"]["boundaries"]["min"], le=conf["d3"]["boundaries"]["max"])
    y: float = Field(ge=conf["d3"]["boundaries"]["min"], le=conf["d3"]["boundaries"]["max"])
    z: float = Field(ge=conf["d3"]["boundaries"]["min"], le=conf["d3"]["boundaries"]["max"])


class Point(Coords3D):
    pass


class Vector(Coords3D):
    pass


class Rotation(BaseModel):
    yaw: float
    roll: float
    pitch: float
